# # -*- coding: utf-8 -*-
#
#    copyright (C) 2023 Steffen Rolapp (github@rolapp.de)
#
#    based on ZattooBoxExtended by Daniel Griner (griner.ch@gmail.com) license under GPL
#
#    This file is part of ZattooHiQ
#
#    zattooHiQ is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    zattooHiQ is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with zattooHiQ.  If not, see <http://www.gnu.org/licenses/>.
#

import xbmc, xbmcgui, xbmcplugin, xbmcaddon, xbmcvfs
import time, datetime, threading
from .zattooDB import ZattooDB

__addon__       = xbmcaddon.Addon()
__addonId__     = __addon__.getAddonInfo('id')
__addonname__   = __addon__.getAddonInfo('name')
__addondir__    = xbmcvfs.translatePath( __addon__.getAddonInfo('profile') )

_zattooDB_ = ZattooDB()


_umlaut_ = {ord('ä'): 'ae', ord('ö'): 'oe', ord('ü'): 'ue', ord('ß'): 'ss'}
localString = __addon__.getLocalizedString
local = xbmc.getLocalizedString
stream_type = __addon__.getSetting('stream_type')
if __addon__.getSetting('show_favourites')=='true':_listMode_ ='favourites'
else: _listMode_ ='all'

# Set Youth Protection PIN
try:
    params = {'password': __addon__.getSetting('password')}
    req = _zattooDB_.zapi.exec_zapiCall('/zapi/account/get_pin', params)
    YPIN = req['pin']
    __addon__.setSetting('ypin', YPIN)
except:
    YPIN = __addon__.getSetting('ypin')

if len(YPIN) != 4:
    xbmcgui.Dialog().ok(__addonname__, localString(31927))
    __addon__.openSettings()

try:
  country=accountData['account']['service_country']
  __addon__.setSetting('country', country)
except: pass

if __addon__.getSetting('country') == 'CH': SWISS = True
else: SWISS = False

DEBUG = __addon__.getSetting('debug')
DOLBY = __addon__.getSetting('dolby')

def debug(content):
    if DEBUG:log(content, xbmc.LOGDEBUG)

def notice(content):
    log(content, xbmc.LOGINFO)

def log(msg, level=xbmc.LOGINFO):
    addon = xbmcaddon.Addon()
    addonID = addon.getAddonInfo('id')
    xbmc.log('%s: %s' % (addonID, msg), level)

def AddonHandle(handle):
    global __addonhandle__
    __addonhandle__ = handle
    
def setplayer(myplayer):
    global player
    player = myplayer
    
def convert_date(date):
    debug(date)
    try:
        res = datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%SZ")
    except TypeError:
        res = datetime.datetime(*(time.strptime(date, "%Y-%m-%dT%H:%M:%SZ")[0:6]))
    res += datetime.timedelta(seconds=_timezone_)
    return str(res.strftime('%A,%d.%B %Y %H:%M'))
    
def convert_date(date):
    debug(date)
    try:
        res = datetime.datetime.strptime(date, "%Y-%m-%dT%H:%M:%SZ")
    except TypeError:
        res = datetime.datetime(*(time.strptime(date, "%Y-%m-%dT%H:%M:%SZ")[0:6]))
    res += datetime.timedelta(seconds=_timezone_)
    return str(res.strftime('%A,%d.%B %Y %H:%M'))

def formatTime(timestamp):
    if timestamp:
        format = xbmc.getRegion('time').replace(':%S', '').replace('%H%H', '%H')
        return timestamp.strftime(format)
    else:
        return ''
  
def formatDate(timestamp, format):
    if timestamp:
        date = timestamp.strftime(format)
        date = date.replace('Monday', local(11))
        date = date.replace('Tuesday', local(12))
        date = date.replace('Wednesday', local(13))
        date = date.replace('Thursday', local(14))
        date = date.replace('Friday', local(15))
        date = date.replace('Saturday', local(16))
        date = date.replace('Sunday', local(17))
        date = date.replace('January', local(21))
        date = date.replace('February', local(22))
        date = date.replace('March', local(23))
        date = date.replace('April', local(24))
        date = date.replace('May', local(25))
        date = date.replace('June', local(26))
        date = date.replace('July', local(27))
        date = date.replace('August', local(28))
        date = date.replace('September', local(29))
        date = date.replace('October', local(30))
        date = date.replace('November', local(31))
        date = date.replace('December', local(32))
        return date
    else:
        return ''

def to_seconds(date):
    return time.mktime(date.timetuple())

def showEpg():
    from .epg import EPG
    currentChannel = _zattooDB_.get_playing()['channel']
    channelList = _zattooDB_.getChannelList(_listMode_ == 'favourites')
    #debug(str(channelList))
    try:
        currentNr=channelList[currentChannel]['nr']
    except:
        currentNr=0
  
    epg = EPG(currentNr)
    epg.loadChannels(_listMode_ == 'favourites')
    epg.show() #doModal()
    while xbmcgui.Window(10000).getProperty('zattoo_runningView')=="epg": xbmc.sleep(200)
    del epg

def watch_channel(channel_id, start, end, showID="", recall='false', add='false'):
    #player = xbmc.Player()
    #_keymap_.toggleKeyMap()
    _zattooDB_=ZattooDB()
    
    #selected currently playing live TV
    playing = _zattooDB_.get_playing()
    xbmcgui.Window(10000).setProperty('lastChannel', playing['channel'])

    pre = __addon__.getSetting('pre_padding')
    post = __addon__.getSetting('post_padding')
    drm = _zattooDB_.get_drm(channel_id)
  
    if recall == 'true':
        debug(recall)
        params = {'stream_type': stream_type, 'enable_eac3':DOLBY, 'pre_padding':pre, 'post_padding':post, 'youth_protection_pin': YPIN}
        if drm: 
            params['stream_type'] = 'dash_widevine'
            params['max_drm_lvl'] = '3'
            if SWISS:
                debug('swiss')
                if platform.system() == 'Linux':
                    #params['quality'] = 'sd'
                    params['stream_type'] = 'dash'
        else:
            params['stream_type'] = 'dash'
        resultData = _zattooDB_.zapi.exec_zapiCall('/zapi/v3/watch/replay/' + channel_id + '/' + showID, params)

        if resultData is None:
            #if add == 'true': return
            xbmcgui.Dialog().notification("ERROR", "NO ZAPI RESULT", channelInfo['logo'], 5000, False)
            return  
        streams = resultData['stream']['watch_urls']
        if len(streams)==0:
            xbmcgui.Dialog().notification("ERROR", "NO STREAM FOUND, CHECK SETTINGS!", channelInfo['logo'], 5000, False)
            return
        # change stream if settings are set
        elif len(streams) > 1 and  __addon__.getSetting('audio_stream') == 'B' and streams[1]['audio_channel'] == 'B': streamNr = 1
        else:  streamNr = 0
        url = streams[streamNr]['url']
    else:
        data = _zattooDB_.get_streams(channel_id) 
        streams = data['streams']
        url = data['url']
        key =data['key']
    # save currently playing
    if showID == '1':
        prog = _zattooDB_.getPrograms({'index':[channel_id]})
        showID = prog[0]['showID']
    program = _zattooDB_.get_showID(showID)
    showinfo = _zattooDB_.getShowInfo(showID)
    channelInfo = _zattooDB_.get_channelInfo(channel_id)
        
    listitem = xbmcgui.ListItem(path=url)      
    if program:
        program = program[0]
        heading = ('[B]' + channelInfo['title'] + '[/B] ').translate(_umlaut_) + '  ' + program['start_date'].strftime('%H:%M') + '-' + program['end_date'].strftime('%H:%M')
        xbmcgui.Dialog().notification(heading, program['title'].translate(_umlaut_), channelInfo['logo'], 5000, False)
    
        #set info for recall
        vinfo = listitem.getVideoInfoTag()
        vinfo.setTvShowTitle(channelInfo['title'])# + ', ' + formatDate(program['start_date'],strftime('%A %H:%M') + '-' + program['end_date'],strftime('%H:%M')))
        vinfo.setTitle(program['title'])
        #vinfo.setResumePoint(0, program['duration'])
        vinfo.setPath(url)
        vinfo.setUniqueIDs({'tvdb':channel_id, 'imdb':showID})
        #meta = { 'title': prog['title'], 'episode': streamNr, 'tvshowtitle': channelInfo['title']+ ', ' + formatDate(prog['start_date'],'%A %H:%M') + '-' + prog['end_date'].strftime('%H:%M'), 'premiered' :'premiered', 'duration' : '20', 'rating': 'rating', 'director': 'director', 'writer': 'writer', 'plot': program['description_long']}
        #listitem.setInfo("Video", meta)
        listitem.setLabel(program['title'] + ' - ' + str(program['description']))
        listitem.setArt({ 'icon': channelInfo['logo'], 'poster': channelInfo['logo'], 'clearlogo' : channelInfo['logo'], 'thumb': channelInfo['logo']})
        listitem.setInfo('Video',{'director': channel_id, 'writer': showID})
    
    if drm:
        listitem.setProperty('inputstream', 'inputstream.adaptive')
        listitem.setProperty('inputstream.adaptive.manifest_type', 'mpd')
        listitem.setProperty('inputstream.adaptive.license_key', key + "||a{SSM}|")
        listitem.setProperty('inputstream.adaptive.license_type', "com.widevine.alpha")
    
    else:
        listitem.setProperty('inputstream', 'inputstream.adaptive')
        listitem.setProperty('inputstream.adaptive.manifest_type', 'mpd')
        
    listitem.setProperty('isplayable', 'true')
    listitem.setIsFolder(False)
      
    xbmcplugin.setResolvedUrl(__addonhandle__, True, listitem)
    # if url.find('-live')>-1 :
        # th = threading.Thread(target=container_refresh)
        # th.start()
        # makeZattooGUI()

  
    #play liveTV: info is created in OSD
    # set add to playlist
    afterRecall = __addon__.getSetting('after_recall')
    xbmcgui.Window(12005).setProperty('after_recall', afterRecall)
  
    #play liveTV: info is created in OSD
    # GreenAir: play live stream or recall without PLAYLIST
    if recall=='false' or afterRecall != '3':
      player.PlayStream(url, listitem)
      # while xbmcgui.getCurrentWindowId() != 12005:
          # xbmc.sleep(200)
      # makeZattooGUI()
      # while (player.isPlaying()): xbmc.sleep(100)

    else:
        PLAYLIST.add(url=streams[streamNr]['url'], listitem=listitem)
        # GreenAir: start playing PLAYLIST if this is the first entry
        if PLAYLIST.size()==1:
            player.PlayStream(PLAYLIST)
            #GreenAir: don't block execution, showToggleImg() has to create zattooPiP on return
            #while player.isPlaying():xbmc.sleep(100)

def container_refresh():
    monitor = xbmc.Monitor()
    while not monitor.abortRequested():
        if monitor.waitForAbort(300): break
        xbmc.executebuiltin('Container.Refresh')      
